@@ -4,6 +4,7 @@ import sys
 import unicodedata
 import itertools
 from functools import partial
+from collections import namedtuple
 
 import numpy as np
 
@@ -12,7 +13,7 @@ from AnyQt.QtGui import (
     QPalette
 )
 from AnyQt.QtWidgets import (
-    QListView, QSizePolicy, QAction, QUndoCommand, QUndoStack, QGridLayout,
+    QSizePolicy, QAction, QUndoCommand, QUndoStack, QGridLayout,
     QFormLayout, QToolButton, QActionGroup
 )
 
@@ -28,7 +29,7 @@ from Orange.widgets.settings import Setting
 from Orange.widgets.utils import itemmodels, colorpalette
 
 from Orange.util import scale, namegen
-from Orange.widgets.widget import OWWidget, Msg
+from Orange.widgets.widget import OWWidget, Msg, Input, Output
 
 
 def indices_to_mask(indices, size):
@@ -71,7 +72,6 @@ def stack_on_condition(a, b, condition):
 # Data manipulation operators
 # ###########################
 
-from collections import namedtuple
 if sys.version_info < (3, 4):
     # use singledispatch backports from pypi
     from singledispatch import singledispatch
@@ -273,8 +273,7 @@ class PutInstanceTool(DataTool):
             event.accept()
             self.editingFinished.emit()
             return True
-        else:
-            return super().mousePressEvent(event)
+        return super().mousePressEvent(event)
 
 
 class PenTool(DataTool):
@@ -286,22 +285,19 @@ class PenTool(DataTool):
             self.editingStarted.emit()
             self.__handleEvent(event)
             return True
-        else:
-            return super().mousePressEvent()
+        return super().mousePressEvent()
 
     def mouseMoveEvent(self, event):
         if event.buttons() & Qt.LeftButton:
             self.__handleEvent(event)
             return True
-        else:
-            return super().mouseMoveEvent()
+        return super().mouseMoveEvent()
 
     def mouseReleaseEvent(self, event):
         if event.button() == Qt.LeftButton:
             self.editingFinished.emit()
             return True
-        else:
-            return super().mouseReleaseEvent()
+        return super().mouseReleaseEvent()
 
     def __handleEvent(self, event):
         pos = self.mapToPlot(event.pos())
@@ -328,23 +324,20 @@ class AirBrushTool(DataTool):
             self.__pos = self.mapToPlot(event.pos())
             self.__timer.start()
             return True
-        else:
-            return super().mousePressEvent(event)
+        return super().mousePressEvent(event)
 
     def mouseMoveEvent(self, event):
         if event.buttons() & Qt.LeftButton:
             self.__pos = self.mapToPlot(event.pos())
             return True
-        else:
-            return super().mouseMoveEvent(event)
+        return super().mouseMoveEvent(event)
 
     def mouseReleaseEvent(self, event):
         if event.button() == Qt.LeftButton:
             self.__timer.stop()
             self.editingFinished.emit()
             return True
-        else:
-            return super().mouseReleaseEvent(event)
+        return super().mouseReleaseEvent(event)
 
     def __timout(self):
         self.issueCommand.emit(
@@ -355,8 +348,7 @@ class AirBrushTool(DataTool):
 def random_state(rstate):
     if isinstance(rstate, np.random.RandomState):
         return rstate
-    else:
-        return np.random.RandomState(rstate)
+    return np.random.RandomState(rstate)
 
 
 def create_data(x, y, radius, size, rstate):
@@ -384,23 +376,20 @@ class MagnetTool(DataTool):
             self._pos = self.mapToPlot(event.pos())
             self.__timer.start()
             return True
-        else:
-            return super().mousePressEvent(event)
+        return super().mousePressEvent(event)
 
     def mouseMoveEvent(self, event):
         if event.buttons() & Qt.LeftButton:
             self._pos = self.mapToPlot(event.pos())
             return True
-        else:
-            return super().mouseMoveEvent(event)
+        return super().mouseMoveEvent(event)
 
     def mouseReleaseEvent(self, event):
         if event.button() == Qt.LeftButton:
             self.__timer.stop()
             self.editingFinished.emit()
             return True
-        else:
-            return super().mouseReleaseEvent(event)
+        return super().mouseReleaseEvent(event)
 
     def __timeout(self):
         self.issueCommand.emit(
@@ -427,23 +416,20 @@ class JitterTool(DataTool):
             self._pos = self.mapToPlot(event.pos())
             self.__timer.start()
             return True
-        else:
-            return super().mousePressEvent(event)
+        return super().mousePressEvent(event)
 
     def mouseMoveEvent(self, event):
         if event.buttons() & Qt.LeftButton:
             self._pos = self.mapToPlot(event.pos())
             return True
-        else:
-            return super().mouseMoveEvent(event)
+        return super().mouseMoveEvent(event)
 
     def mouseReleaseEvent(self, event):
         if event.button() == Qt.LeftButton:
             self.__timer.stop()
             self.editingFinished.emit()
             return True
-        else:
-            return super().mouseReleaseEvent(event)
+        return super().mouseReleaseEvent(event)
 
     def _do(self):
         self.issueCommand.emit(
@@ -507,8 +493,7 @@ class SelectTool(DataTool):
             self.setSelectionRect(QRectF(pos, pos))
             event.accept()
             return True
-        else:
-            return super().mousePressEvent(event)
+        return super().mousePressEvent(event)
 
     def mouseMoveEvent(self, event):
         if event.buttons() & Qt.LeftButton:
@@ -516,8 +501,7 @@ class SelectTool(DataTool):
             self.setSelectionRect(QRectF(self._start_pos, pos).normalized())
             event.accept()
             return True
-        else:
-            return super().mouseMoveEvent(event)
+        return super().mouseMoveEvent(event)
 
     def mouseReleaseEvent(self, event):
         if event.button() == Qt.LeftButton:
@@ -528,8 +512,7 @@ class SelectTool(DataTool):
             self._item.setCursor(Qt.OpenHandCursor)
             self._mouse_dragging = False
             return True
-        else:
-            return super().mouseReleaseEvent(event)
+        return super().mouseReleaseEvent(event)
 
     def activate(self):
         if self._item is None:
@@ -647,8 +630,7 @@ def indices_eq(ind1, ind2):
     if isinstance(ind1, tuple) and isinstance(ind2, tuple):
         if len(ind1) != len(ind2):
             return False
-        else:
-            return all(indices_eq(i1, i2) for i1, i2 in zip(ind1, ind2))
+        return all(indices_eq(i1, i2) for i1, i2 in zip(ind1, ind2))
     elif isinstance(ind1, slice) and isinstance(ind2, slice):
         return ind1 == ind2
     elif ind1 is ... and ind2 is ...:
@@ -658,8 +640,7 @@ def indices_eq(ind1, ind2):
 
     if ind1.shape != ind2.shape or ind1.dtype != ind2.dtype:
         return False
-    else:
-        return (ind1 == ind2).all()
+    return (ind1 == ind2).all()
 
 
 def merge_cmd(composit):
@@ -739,24 +720,23 @@ class ColoredListModel(itemmodels.PyListModel):
                 role == Qt.DecorationRole and \
                 0 <= index.row() < self.colors.number_of_colors:
             return gui.createAttributePixmap("", self.colors[index.row()])
-        else:
-            return super().data(index, role)
+        return super().data(index, role)
 
 
-def _i(name, icon_path="icons/paintdata",
-       widg_path=os.path.dirname(os.path.abspath(__file__))):
+def _icon(name, icon_path="icons/paintdata",
+        widg_path=os.path.dirname(os.path.abspath(__file__))):
     return os.path.join(widg_path, icon_path, name)
 
 
 class OWPaintData(OWWidget):
     TOOLS = [
-        ("Brush", "Create multiple instances", AirBrushTool, _i("brush.svg")),
-        ("Put", "Put individual instances", PutInstanceTool, _i("put.svg")),
+        ("Brush", "Create multiple instances", AirBrushTool, _icon("brush.svg")),
+        ("Put", "Put individual instances", PutInstanceTool, _icon("put.svg")),
         ("Select", "Select and move instances", SelectTool,
-            _i("select-transparent_42px.png")),
-        ("Jitter", "Jitter instances", JitterTool, _i("jitter.svg")),
-        ("Magnet", "Attract multiple instances", MagnetTool, _i("magnet.svg")),
-        ("Clear", "Clear the plot", ClearTool, _i("../../../icons/Dlg_clear.png"))
+         _icon("select-transparent_42px.png")),
+        ("Jitter", "Jitter instances", JitterTool, _icon("jitter.svg")),
+        ("Magnet", "Attract multiple instances", MagnetTool, _icon("magnet.svg")),
+        ("Clear", "Clear the plot", ClearTool, _icon("../../../icons/Dlg_clear.png"))
     ]
 
     name = "Paint Data"
@@ -765,10 +745,13 @@ class OWPaintData(OWWidget):
     priority = 15
     keywords = ["data", "paint", "create"]
 
-    outputs = [("Data", Orange.data.Table)]
-    inputs = [("Data", Orange.data.Table, "set_data")]
+    class Inputs:
+        data = Input("Data", Orange.data.Table)
 
-    autocommit = Setting(False)
+    class Outputs:
+        data = Output("Data", Orange.data.Table)
+
+    autocommit = Setting(True)
     table_name = Setting("Painted data")
     attr1 = Setting("x")
     attr2 = Setting("y")
@@ -778,12 +761,14 @@ class OWPaintData(OWWidget):
     density = Setting(7)
     #: current data array (shape=(N, 3)) as presented on the output
     data = Setting(None, schema_only=True)
+    labels = Setting(["C1", "C2"], schema_only=True)
 
     graph_name = "plot"
 
     class Warning(OWWidget.Warning):
         no_input_variables = Msg("Input data has no variables")
         continuous_target = Msg("Continuous target value can not be used.")
+        sparse_not_supported = Msg("Sparse data is ignored.")
 
     class Information(OWWidget.Information):
         use_first_two = \
@@ -803,22 +788,26 @@ class OWPaintData(OWWidget):
         #: a copy of this array (as seen when the `invalidate` method is
         #: called
         self.__buffer = None
-        self.labels = ["C1", "C2"]
 
         self.undo_stack = QUndoStack(self)
 
         self.class_model = ColoredListModel(
             self.labels, self,
             flags=Qt.ItemIsSelectable | Qt.ItemIsEnabled |
-                  Qt.ItemIsEditable)
+            Qt.ItemIsEditable)
 
         self.class_model.dataChanged.connect(self._class_value_changed)
         self.class_model.rowsInserted.connect(self._class_count_changed)
         self.class_model.rowsRemoved.connect(self._class_count_changed)
 
-        if self.data is None:
-            self.data = np.zeros((0, 3))
-        self.__buffer = self.data.copy()
+        if not self.data:
+            self.data = []
+            self.__buffer = np.zeros((0, 3))
+        elif isinstance(self.data, np.ndarray):
+            self.__buffer = self.data.copy()
+            self.data = self.data.tolist()
+        else:
+            self.__buffer = np.array(self.data)
 
         self.colors = colorpalette.ColorPaletteGenerator(
             len(colorpalette.DefaultRGBColors))
@@ -846,11 +835,8 @@ class OWPaintData(OWWidget):
         gui.separator(namesBox)
 
         gui.widgetLabel(namesBox, "Labels")
-        self.classValuesView = listView = QListView(
-            selectionMode=QListView.SingleSelection,
-            sizePolicy=QSizePolicy(QSizePolicy.Ignored,
-                                   QSizePolicy.Maximum)
-        )
+        self.classValuesView = listView = gui.ListViewWithSizeHint(
+            preferred_size=(-1, 30))
         listView.setModel(self.class_model)
         itemmodels.select_row(listView, 0)
         namesBox.layout().addWidget(listView)
@@ -940,7 +926,6 @@ class OWPaintData(OWWidget):
             tBox, self, "Reset to Input Data", self.reset_to_input)
         self.btResetToInput.setDisabled(True)
 
-        gui.rubber(self.controlArea)
         gui.auto_commit(self.left_side, self, "autocommit",
                         "Send")
 
@@ -993,10 +978,14 @@ class OWPaintData(OWWidget):
             if tool.only2d:
                 button.setDisabled(not self.hasAttr2)
 
+    @Inputs.data
     def set_data(self, data):
         """Set the input_data and call reset_to_input"""
         def _check_and_set_data(data):
             self.clear_messages()
+            if data and data.is_sparse():
+                self.Warning.sparse_not_supported()
+                return False
             if data is not None and len(data):
                 if not data.domain.attributes:
                     self.Warning.no_input_variables()
@@ -1053,8 +1042,8 @@ class OWPaintData(OWWidget):
         newindex = min(max(index, 0), len(self.class_model) - 1)
         itemmodels.select_row(self.classValuesView, newindex)
 
-        self.data = self.input_data
-        self.__buffer = self.data.copy()
+        self.data = self.input_data.tolist()
+        self.__buffer = self.input_data.copy()
 
         prev_attr2 = self.hasAttr2
         self.hasAttr2 = self.input_has_attr2
@@ -1122,8 +1111,7 @@ class OWPaintData(OWWidget):
         rows = self.classValuesView.selectedIndexes()
         if rows:
             return rows[0].row()
-        else:
-            return None
+        return None
 
     def set_current_tool(self, tool):
         prev_tool = self.current_tool.__class__
@@ -1176,14 +1164,14 @@ class OWPaintData(OWWidget):
         name = "Name"
 
         if (not self.hasAttr2 and
-            isinstance(cmd, (Move, MoveSelection, Jitter, Magnet))):
+                isinstance(cmd, (Move, MoveSelection, Jitter, Magnet))):
             # tool only supported if both x and y are enabled
             return
 
         if isinstance(cmd, Append):
             cls = self.selected_class_label()
             points = np.array([(p.x(), p.y() if self.hasAttr2 else 0, cls)
-                                  for p in cmd.points])
+                               for p in cmd.points])
             self.undo_stack.push(UndoCommand(Append(points), self, text=name))
         elif isinstance(cmd, Move):
             self.undo_stack.push(UndoCommand(cmd, self, text=name))
@@ -1258,19 +1246,20 @@ class OWPaintData(OWWidget):
         self.invalidate()
 
     def invalidate(self):
-        self.data = self.__buffer.copy()
+        self.data = self.__buffer.tolist()
         self.commit()
 
     def commit(self):
-        if len(self.data) == 0:
-            self.send("Data", None)
+        data = np.array(self.data)
+        if len(data) == 0:
+            self.Outputs.data.send(None)
             return
         if self.hasAttr2:
-            X, Y = self.data[:, :2], self.data[:, 2]
+            X, Y = data[:, :2], data[:, 2]
             attrs = (Orange.data.ContinuousVariable(self.attr1),
                      Orange.data.ContinuousVariable(self.attr2))
         else:
-            X, Y = self.data[:, np.newaxis, 0], self.data[:, 2]
+            X, Y = data[:, np.newaxis, 0], data[:, 2]
             attrs = (Orange.data.ContinuousVariable(self.attr1),)
         if len(np.unique(Y)) >= 2:
             domain = Orange.data.Domain(
@@ -1283,11 +1272,11 @@ class OWPaintData(OWWidget):
             domain = Orange.data.Domain(attrs)
             data = Orange.data.Table.from_numpy(domain, X)
         data.name = self.table_name
-        self.send("Data", data)
+        self.Outputs.data.send(data)
 
     def sizeHint(self):
         sh = super().sizeHint()
-        return sh.expandedTo(QSize(1200, 800))
+        return sh.expandedTo(QSize(570, 690))
 
     def onDeleteWidget(self):
         self.plot.clear()
@@ -1302,7 +1291,7 @@ class OWPaintData(OWWidget):
         self.report_items("Painted data", settings)
         self.report_plot()
 
-def test():
+def main():
     from AnyQt.QtWidgets import QApplication
     import gc
     import sip
@@ -1321,4 +1310,4 @@ def test():
 
 
 if __name__ == "__main__":
-    sys.exit(test())
+    sys.exit(main())
