@@ -6,33 +6,35 @@ import itertools
 import unicodedata
 from unittest.mock import patch
 
-from PyQt4 import QtGui, QtCore
-
-from PyQt4.QtGui import (
-    QTextCursor, QFont, QColor, QPalette, QListView, QSizePolicy, QAction,
-    QMenu, QKeySequence, QSplitter, QToolButton, QItemSelectionModel,
-    QFileDialog
+from AnyQt.QtWidgets import (
+    QPlainTextEdit, QListView, QSizePolicy, QMenu, QSplitter, QLineEdit,
+    QAction, QToolButton, QFileDialog, QStyledItemDelegate,
+    QStyleOptionViewItem, QPlainTextDocumentLayout
 )
-
-from PyQt4.QtCore import Qt, QRegExp, QByteArray
+from AnyQt.QtGui import (
+    QColor, QBrush, QPalette, QFont, QTextDocument,
+    QSyntaxHighlighter, QTextCharFormat, QTextCursor, QKeySequence,
+)
+from AnyQt.QtCore import Qt, QRegExp, QByteArray, QItemSelectionModel
 
 import Orange.data
 from Orange.base import Learner, Model
 from Orange.widgets import widget, gui
 from Orange.widgets.utils import itemmodels
 from Orange.widgets.settings import Setting
+from Orange.widgets.widget import OWWidget, Input, Output
 
 __all__ = ["OWPythonScript"]
 
 
 def text_format(foreground=Qt.black, weight=QFont.Normal):
-    fmt = QtGui.QTextCharFormat()
-    fmt.setForeground(QtGui.QBrush(foreground))
+    fmt = QTextCharFormat()
+    fmt.setForeground(QBrush(foreground))
     fmt.setFontWeight(weight)
     return fmt
 
 
-class PythonSyntaxHighlighter(QtGui.QSyntaxHighlighter):
+class PythonSyntaxHighlighter(QSyntaxHighlighter):
     def __init__(self, parent=None):
 
         self.keywordFormat = text_format(Qt.blue, QFont.Bold)
@@ -93,7 +95,7 @@ class PythonSyntaxHighlighter(QtGui.QSyntaxHighlighter):
                                 3)
 
 
-class PythonScriptEditor(QtGui.QPlainTextEdit):
+class PythonScriptEditor(QPlainTextEdit):
     INDENT = 4
 
     def lastLine(self):
@@ -119,7 +121,7 @@ class PythonScriptEditor(QtGui.QPlainTextEdit):
             text = self.lastLine()
             if text and not text.strip():
                 cursor = self.textCursor()
-                for i in range(min(self.INDENT, len(text))):
+                for _ in range(min(self.INDENT, len(text))):
                     cursor.deletePreviousChar()
             else:
                 super().keyPressEvent(event)
@@ -128,9 +130,9 @@ class PythonScriptEditor(QtGui.QPlainTextEdit):
             super().keyPressEvent(event)
 
 
-class PythonConsole(QtGui.QPlainTextEdit, code.InteractiveConsole):
+class PythonConsole(QPlainTextEdit, code.InteractiveConsole):
     def __init__(self, locals=None, parent=None):
-        QtGui.QPlainTextEdit.__init__(self, parent)
+        QPlainTextEdit.__init__(self, parent)
         code.InteractiveConsole.__init__(self, locals)
         self.history, self.historyInd = [""], 0
         self.loop = self.interact()
@@ -230,9 +232,9 @@ class PythonConsole(QtGui.QPlainTextEdit, code.InteractiveConsole):
             self.complete()
         elif event.key() in [Qt.Key_Left, Qt.Key_Backspace]:
             if self.textCursor().position() > self.newPromptPos:
-                QtGui.QPlainTextEdit.keyPressEvent(self, event)
+                QPlainTextEdit.keyPressEvent(self, event)
         else:
-            QtGui.QPlainTextEdit.keyPressEvent(self, event)
+            QPlainTextEdit.keyPressEvent(self, event)
 
     def historyUp(self):
         self.setLine(self.history[self.historyInd])
@@ -310,7 +312,7 @@ class Script(object):
         self.filename = filename
 
 
-class ScriptItemDelegate(QtGui.QStyledItemDelegate):
+class ScriptItemDelegate(QStyledItemDelegate):
     def __init__(self, parent):
         super().__init__(parent)
 
@@ -324,13 +326,13 @@ class ScriptItemDelegate(QtGui.QStyledItemDelegate):
         script = index.data(Qt.DisplayRole)
 
         if script.flags & Script.Modified:
-            option = QtGui.QStyleOptionViewItemV4(option)
+            option = QStyleOptionViewItem(option)
             option.palette.setColor(QPalette.Text, QColor(Qt.red))
             option.palette.setColor(QPalette.Highlight, QColor(Qt.darkRed))
         super().paint(painter, option, index)
 
     def createEditor(self, parent, option, index):
-        return QtGui.QLineEdit(parent)
+        return QLineEdit(parent)
 
     def setEditorData(self, editor, index):
         script = index.data(Qt.DisplayRole)
@@ -355,27 +357,32 @@ class OWPythonScript(widget.OWWidget):
     icon = "icons/PythonScript.svg"
     priority = 3150
 
-    inputs = [("in_data", Orange.data.Table, "setExampleTable",
-               widget.Default),
-#               ("in_distance", Orange.misc.SymMatrix, "setDistanceMatrix",
-#                widget.Default),
-              ("in_learner", Learner, "setLearner",
-               widget.Default),
-              ("in_classifier", Model, "setClassifier",
-               widget.Default),
-              ("in_object", object, "setObject")]
+    inputs_names = ["in_data", "in_learner", "in_classifier", "in_object"]
 
-    outputs = [("out_data", Orange.data.Table, ),
-#                ("out_distance", Orange.misc.SymMatrix, ),
-               ("out_learner", Learner, ),
-               ("out_classifier", Model, widget.Dynamic),
-               ("out_object", object, widget.Dynamic)]
+    class Inputs:
+        in_data = Input("in_data", Orange.data.Table, default=True)
+        # in_distance = Input("in_distance", Orange.misc.SymMatrix, default=True)
+        in_learner = Input("in_learner", Learner, default=True)
+        in_classifier = Input("in_classifier", Model, default=True)
+        in_object = Input("in_object", object)
+
+    outputs_names = ["out_data", "out_learner", "out_classifier", "out_object"]
+
+    class Outputs:
+        out_data = Output("out_data", Orange.data.Table)
+        # out_distance = Output(("out_distance", Orange.misc.SymMatrix)
+        out_learner = Output("out_learner", Learner)
+        out_classifier = Output("out_classifier", Model)
+        out_object = Output("out_object", object)
 
     libraryListSource = \
         Setting([Script("Hello world", "print('Hello world')\n")])
     currentScriptIndex = Setting(0)
     splitterState = Setting(None)
-    auto_execute = Setting(False)
+    auto_execute = Setting(True)
+
+    class Error(OWWidget.Error):
+        pass
 
     def __init__(self):
         super().__init__()
@@ -385,7 +392,6 @@ class OWPythonScript(widget.OWWidget):
         self.in_learner = None
         self.in_classifier = None
         self.in_object = None
-        self.auto_execute = False
 
         for s in self.libraryListSource:
             s.flags = 0
@@ -396,9 +402,9 @@ class OWPythonScript(widget.OWWidget):
         gui.label(
             self.infoBox, self,
             "<p>Execute python script.</p><p>Input variables:<ul><li> " + \
-            "<li>".join(t.name for t in self.inputs) + \
+            "<li>".join(name for name in self.inputs_names) + \
             "</ul></p><p>Output variables:<ul><li>" + \
-            "<li>".join(t.name for t in self.outputs) + \
+            "<li>".join(name for name in self.outputs_names) + \
             "</ul></p>"
         )
 
@@ -413,7 +419,7 @@ class OWPythonScript(widget.OWWidget):
 
         self.libraryView = QListView(
             editTriggers=QListView.DoubleClicked |
-                         QListView.EditKeyPressed,
+            QListView.EditKeyPressed,
             sizePolicy=QSizePolicy(QSizePolicy.Ignored,
                                    QSizePolicy.Preferred)
         )
@@ -507,18 +513,23 @@ class OWPythonScript(widget.OWWidget):
         self.controlArea.layout().addStretch(1)
         self.resize(800, 600)
 
+    @Inputs.in_data
     def setExampleTable(self, et):
         self.in_data = et
 
+    # @Inputs.in_distance
     def setDistanceMatrix(self, dm):
         self.in_distance = dm
 
+    @Inputs.in_learner
     def setLearner(self, learner):
         self.in_learner = learner
 
+    @Inputs.in_classifier
     def setClassifier(self, classifier):
         self.in_classifier = classifier
 
+    @Inputs.in_object
     def setObject(self, obj):
         self.in_object = obj
 
@@ -540,16 +551,16 @@ class OWPythonScript(widget.OWWidget):
         self.setSelectedScript(len(self.libraryList) - 1)
 
     def onAddScriptFromFile(self, *args):
-        filename = QFileDialog.getOpenFileName(
+        filename, _ = QFileDialog.getOpenFileName(
             self, 'Open Python Script',
             os.path.expanduser("~/"),
             'Python files (*.py)\nAll files(*.*)'
         )
-
-        filename = str(filename)
         if filename:
             name = os.path.basename(filename)
-            contents = open(filename, "rb").read().decode("utf-8", errors="ignore")
+            # TODO: use `tokenize.detect_encoding`
+            with open(filename, encoding="utf-8") as f:
+                contents = f.read()
             self.libraryList.append(Script(name, contents, 0, filename))
             self.setSelectedScript(len(self.libraryList) - 1)
 
@@ -580,8 +591,8 @@ class OWPythonScript(widget.OWWidget):
             script = self.libraryList[script]
 
         if script not in self._cachedDocuments:
-            doc = QtGui.QTextDocument(self)
-            doc.setDocumentLayout(QtGui.QPlainTextDocumentLayout(doc))
+            doc = QTextDocument(self)
+            doc.setDocumentLayout(QPlainTextDocumentLayout(doc))
             doc.setPlainText(script.script)
             doc.setDefaultFont(QFont(self.defaultFont))
             doc.highlighter = PythonSyntaxHighlighter(doc)
@@ -604,7 +615,7 @@ class OWPythonScript(widget.OWWidget):
             self.libraryList.emitDataChanged(index)
 
     def onSpliterMoved(self, pos, ind):
-        self.splitterState = str(self.splitCanvas.saveState())
+        self.splitterState = bytes(self.splitCanvas.saveState())
 
     def updateSelecetdScriptState(self):
         index = self.selectedScriptIndex()
@@ -622,7 +633,7 @@ class OWPythonScript(widget.OWWidget):
         else:
             filename = os.path.expanduser("~/")
 
-        filename = QFileDialog.getSaveFileName(
+        filename, _ = QFileDialog.getSaveFileName(
             self, 'Save Python Script',
             filename,
             'Python files (*.py)\nAll files(*.*)'
@@ -641,11 +652,12 @@ class OWPythonScript(widget.OWWidget):
             f.close()
 
     def initial_locals_state(self):
-        d = dict([(i.name, getattr(self, i.name, None)) for i in self.inputs])
-        d.update(dict([(o.name, None) for o in self.outputs]))
+        d = dict([(name, getattr(self, name, None)) for name in self.inputs_names])
+        d.update(dict([(name, None) for name in self.outputs_names]))
         return d
 
     def commit(self):
+        self.Error.clear()
         self._script = str(self.text.toPlainText())
         lcls = self.initial_locals_state()
         lcls["_script"] = str(self.text.toPlainText())
@@ -653,13 +665,21 @@ class OWPythonScript(widget.OWWidget):
         self.console.write("\nRunning script:\n")
         self.console.push("exec(_script)")
         self.console.new_prompt(sys.ps1)
-        for out in self.outputs:
-            signal = out.name
-            self.send(signal, self.console.locals.get(signal, None))
+        for signal in self.outputs_names:
+            out_var = self.console.locals.get(signal, None)
+            signal_type = getattr(self.Outputs, signal).type
+            if not isinstance(out_var, signal_type) and out_var is not None:
+                self.Error.add_message(signal,
+                                       "Variable '{}' has to be an instance of '{}'.".
+                                       format(signal, signal_type.__name__))
+                getattr(self.Error, signal)()
+                out_var = None
+            getattr(self.Outputs, signal).send(out_var)
 
 
 if __name__ == "__main__":
-    app = QtGui.QApplication(sys.argv)
+    from AnyQt.QtWidgets import QApplication
+    app = QApplication(sys.argv)
     ow = OWPythonScript()
     ow.show()
     app.exec_()
