@@ -1,14 +1,14 @@
+from contextlib import contextmanager
 import os
 import time
-from contextlib import contextmanager
 import unittest
 from unittest.mock import Mock
 
+import numpy as np
 import sip
 
-import numpy as np
 from AnyQt.QtCore import Qt
-from AnyQt.QtTest import QTest
+from AnyQt.QtTest import QTest, QSignalSpy
 from AnyQt.QtWidgets import (
     QApplication, QComboBox, QSpinBox, QDoubleSpinBox, QSlider
 )
@@ -34,6 +34,8 @@ sip.setdestroyonexit(False)
 
 app = None
 
+DEFAULT_TIMEOUT = 5000
+
 
 class DummySignalManager:
     def __init__(self):
@@ -88,6 +90,10 @@ class WidgetTest(GuiTest):
         cls.widgets.append(report)
         OWReport.get_instance = lambda: report
 
+    def tearDown(self):
+        """Process any pending events before the next test is executed."""
+        self.process_events()
+
     def create_widget(self, cls, stored_settings=None, reset_default_settings=True):
         """Create a widget instance using mock signal_manager.
 
@@ -141,8 +147,7 @@ class WidgetTest(GuiTest):
             # Reset context settings
             settings_handler.global_contexts = []
 
-    @staticmethod
-    def process_events(until: callable=None):
+    def process_events(self, until: callable=None, timeout=DEFAULT_TIMEOUT):
         """Process Qt events, optionally until `until` returns
         something True-ish.
 
@@ -153,6 +158,9 @@ class WidgetTest(GuiTest):
         until: callable or None
             If callable, the events are processed until the function returns
             something True-ish.
+        timeout: int
+            If until condition is not satisfied within timeout milliseconds,
+            a TimeoutError is raised.
 
         Returns
         -------
@@ -161,6 +169,7 @@ class WidgetTest(GuiTest):
         if until is None:
             until = lambda: True
 
+        started = time.clock()
         while True:
             app.processEvents()
             try:
@@ -169,6 +178,8 @@ class WidgetTest(GuiTest):
                     return result
             except Exception:  # until can fail with anything; pylint: disable=broad-except
                 pass
+            if (time.clock() - started) * 1000 > timeout:
+                raise TimeoutError()
             time.sleep(.05)
 
     def show(self, widget=None):
@@ -180,7 +191,7 @@ class WidgetTest(GuiTest):
         widget.show()
         app.exec()
 
-    def send_signal(self, input_name, value, *args, widget=None):
+    def send_signal(self, input, value, *args, widget=None, wait=-1):
         """ Send signal to widget by calling appropriate triggers.
 
         Parameters
@@ -191,19 +202,26 @@ class WidgetTest(GuiTest):
             channel id, used for inputs with flag Multiple
         widget : Optional[OWWidget]
             widget to send signal to. If not set, self.widget is used
+        wait : int
+            The amount of time to wait for the widget to complete.
         """
         if widget is None:
             widget = self.widget
-        for input_signal in widget.inputs:
-            if input_signal.name == input_name:
-                getattr(widget, input_signal.handler)(value, *args)
-                break
-        else:
-            raise ValueError("'{}' is not an input name for widget {}"
-                             .format(input_name, type(widget).__name__))
+        if isinstance(input, str):
+            for input_signal in widget.get_signals("inputs"):
+                if input_signal.name == input:
+                    input = input_signal
+                    break
+            else:
+                raise ValueError("'{}' is not an input name for widget {}"
+                                 .format(input, type(widget).__name__))
+        getattr(widget, input.handler)(value, *args)
         widget.handleNewSignals()
+        if wait >= 0 and widget.isBlocking():
+            spy = QSignalSpy(widget.blockingStateChanged)
+            self.assertTrue(spy.wait(timeout=wait))
 
-    def get_output(self, output_name, widget=None):
+    def get_output(self, output, widget=None):
         """Return the last output that has been sent from the widget.
 
         Parameters
@@ -218,7 +236,9 @@ class WidgetTest(GuiTest):
         """
         if widget is None:
             widget = self.widget
-        return self.signal_manager.outputs.get((widget, output_name), None)
+        if not isinstance(output, str):
+            output = output.name
+        return self.signal_manager.outputs.get((widget, output), None)
 
     @contextmanager
     def modifiers(self, modifiers):
@@ -237,6 +257,13 @@ class WidgetTest(GuiTest):
             QTest.keyRelease(self.widget, Qt.Key_BassBoost, old_modifiers)
 
 
+class TestWidgetTest(WidgetTest):
+    """Meta tests for widget test helpers"""
+
+    def test_process_events_handles_timeouts(self):
+        with self.assertRaises(TimeoutError):
+            self.process_events(until=lambda: False, timeout=0)
+
 
 
 class BaseParameterMapping:
@@ -426,7 +453,7 @@ class WidgetLearnerTestMixin:
         self.widget.apply_button.button.click()
         self.send_signal("Data", None)
         self.assertEqual(self.widget.data, None)
-        self.assertIsNone(self.get_output(self.model_name))
+        self.assertIsNone(self.get_output(self.widget.Outputs.model))
 
     def test_input_data_learner_adequacy(self):
         """Check if error message is shown with inadequate data on input"""
@@ -482,12 +509,12 @@ class WidgetLearnerTestMixin:
 
     def test_output_model(self):
         """Check if model is on output after sending data and apply"""
-        self.assertIsNone(self.get_output(self.model_name))
+        self.assertIsNone(self.get_output(self.widget.Outputs.model))
         self.widget.apply_button.button.click()
-        self.assertIsNone(self.get_output(self.model_name))
+        self.assertIsNone(self.get_output(self.widget.Outputs.model))
         self.send_signal('Data', self.data)
         self.widget.apply_button.button.click()
-        model = self.get_output(self.model_name)
+        model = self.get_output(self.widget.Outputs.model)
         self.assertIsNotNone(model)
         self.assertIsInstance(model, self.widget.LEARNER.__returns__)
         self.assertIsInstance(model, self.model_class)
@@ -508,7 +535,7 @@ class WidgetLearnerTestMixin:
         self.widget.name_line_edit.setText(new_name)
         self.send_signal("Data", self.data)
         self.widget.apply_button.button.click()
-        self.assertEqual(self.get_output(self.model_name).name, new_name)
+        self.assertEqual(self.get_output(self.widget.Outputs.model).name, new_name)
 
     def _get_param_value(self, learner, param):
         if isinstance(learner, Fitter):
@@ -565,7 +592,7 @@ class WidgetLearnerTestMixin:
                         "Mismatching setting for parameter '%s'" % parameter)
 
                     if issubclass(self.widget.LEARNER, SklModel):
-                        model = self.get_output(self.model_name)
+                        model = self.get_output(self.widget.Outputs.model)
                         if model is not None:
                             self.assertEqual(self._get_param_value(model, parameter), value)
                             self.assertFalse(self.widget.Error.active)
@@ -628,14 +655,12 @@ class WidgetOutputsTestMixin:
         self.data = Table("iris")
         self.same_input_output_domain = True
 
-    def test_outputs(self):
+    def test_outputs(self, timeout=DEFAULT_TIMEOUT):
         self.send_signal(self.signal_name, self.signal_data)
 
-        # only needed in TestOWMDS
-        if type(self).__name__ == "TestOWMDS":
-            from AnyQt.QtCore import QEvent
-            self.widget.customEvent(QEvent(QEvent.User))
-            self.widget.commit()
+        if self.widget.isBlocking():
+            spy = QSignalSpy(self.widget.blockingStateChanged)
+            self.assertTrue(spy.wait(timeout))
 
         # check selected data output
         self.assertIsNone(self.get_output("Selected Data"))
