@@ -1,10 +1,14 @@
+# pylint: disable=protected-access
 from contextlib import contextmanager
 import os
 import pickle
-from tempfile import mkstemp
+from tempfile import mkstemp, NamedTemporaryFile
+
 import unittest
 from unittest.mock import patch, Mock
 import warnings
+
+from Orange.tests import named_file
 from Orange.widgets.settings import SettingsHandler, Setting, SettingProvider,\
     VERSION_KEY, rename_setting, Context, migrate_str_to_variable
 
@@ -64,6 +68,33 @@ class SettingHandlerTestCase(unittest.TestCase):
 
         os.remove(settings_file)
 
+    def test_write_defaults_handles_permission_error(self):
+        handler = SettingsHandler()
+
+        with named_file("") as f:
+            handler._get_settings_filename = lambda: f
+
+            with patch('Orange.widgets.settings.open', create=True) as mocked_open:
+                mocked_open.side_effect = PermissionError()
+
+                handler.write_defaults()
+
+    def test_write_defaults_handles_writing_errors(self):
+        handler = SettingsHandler()
+
+        for error in (EOFError, IOError, pickle.PicklingError):
+            f = NamedTemporaryFile("wt", delete=False)
+            f.close()  # so it can be opened on windows
+            handler._get_settings_filename = lambda x=f: x.name
+
+            with patch.object(handler, "write_defaults_file") as mocked_write:
+                mocked_write.side_effect = error()
+
+                handler.write_defaults()
+
+            # Corrupt setting files should be removed
+            self.assertFalse(os.path.exists(f.name))
+
     def test_initialize_widget(self):
         handler = SettingsHandler()
         handler.defaults = {'default': 42, 'setting': 1}
@@ -266,6 +297,19 @@ class SettingHandlerTestCase(unittest.TestCase):
         settings = handler.pack_data(widget)
         self.assertIn(VERSION_KEY, settings)
 
+    def test_initialize_copies_mutables(self):
+        handler = SettingsHandler()
+        handler.bind(SimpleWidget)
+        handler.defaults = dict(list_setting=[])
+
+        widget = SimpleWidget()
+        handler.initialize(widget)
+
+        widget2 = SimpleWidget()
+        handler.initialize(widget2)
+
+        self.assertNotEqual(id(widget.list_setting), id(widget2.list_setting))
+
     @contextmanager
     def override_default_settings(self, widget, defaults=None):
         if defaults is None:
@@ -273,9 +317,9 @@ class SettingHandlerTestCase(unittest.TestCase):
 
         h = SettingsHandler()
         h.widget_class = widget
+        h.defaults = defaults
         filename = h._get_settings_filename()
-        with open(filename, "wb") as f:
-            pickle.dump(defaults, f)
+        h.write_defaults()
 
         yield
 
@@ -292,6 +336,7 @@ class SimpleWidget:
 
     setting = Setting(42)
     schema_only_setting = Setting(None, schema_only=True)
+    list_setting = Setting([])
     non_setting = 5
 
     component = SettingProvider(Component)
