@@ -1,5 +1,4 @@
 import bottleneck as bn
-import numpy
 from AnyQt.QtCore import Qt
 from scipy.sparse import issparse
 
@@ -8,12 +7,17 @@ import Orange.misc
 from Orange import distance
 from Orange.widgets import gui, settings
 from Orange.widgets.utils.sql import check_sql_input
-from Orange.widgets.widget import OWWidget, Msg
+from Orange.widgets.widget import OWWidget, Msg, Input, Output
+
+# A placeholder. This metric is handled specially in commit method.
+__Mahalanobis = distance.MahalanobisDistance()
+__Mahalanobis.fit = None
+
 
 METRICS = [
     distance.Euclidean,
     distance.Manhattan,
-    distance.Mahalanobis,
+    __Mahalanobis,
     distance.Cosine,
     distance.Jaccard,
     distance.SpearmanR,
@@ -28,8 +32,11 @@ class OWDistances(OWWidget):
     description = "Compute a matrix of pairwise distances."
     icon = "icons/Distance.svg"
 
-    inputs = [("Data", Orange.data.Table, "set_data")]
-    outputs = [("Distances", Orange.misc.DistMatrix)]
+    class Inputs:
+        data = Input("Data", Orange.data.Table)
+
+    class Outputs:
+        distances = Output("Distances", Orange.misc.DistMatrix, dynamic=False)
 
     axis = settings.Setting(0)
     metric_idx = settings.Setting(0)
@@ -39,13 +46,15 @@ class OWDistances(OWWidget):
     buttons_area_orientation = Qt.Vertical
 
     class Error(OWWidget.Error):
-        no_continuous_features = Msg("No continuous features")
+        no_continuous_features = Msg("No numeric features")
         dense_metric_sparse_data = Msg("Selected metric does not support sparse data")
         empty_data = Msg("Empty data set")
         mahalanobis_error = Msg("{}")
+        distances_memory_error = Msg("Not enough memory.")
+        distances_value_error = Msg("Error occurred while calculating distances\n{}")
 
     class Warning(OWWidget.Warning):
-        ignoring_discrete = Msg("Ignoring discrete features")
+        ignoring_discrete = Msg("Ignoring categorical features")
         imputing_data = Msg("Imputing missing values")
 
     def __init__(self):
@@ -55,12 +64,12 @@ class OWDistances(OWWidget):
 
         gui.radioButtons(self.controlArea, self, "axis", ["Rows", "Columns"],
                          box="Distances between", callback=self._invalidate
-        )
+                        )
         self.metrics_combo = gui.comboBox(self.controlArea, self, "metric_idx",
                                           box="Distance Metric",
                                           items=[m.name for m in METRICS],
                                           callback=self._invalidate
-        )
+                                         )
         box = gui.auto_commit(self.buttonsArea, self, "autocommit", "Apply",
                               box=False, checkbox_label="Apply automatically")
         box.layout().insertWidget(0, self.report_button)
@@ -68,6 +77,7 @@ class OWDistances(OWWidget):
 
         self.layout().setSizeConstraint(self.layout().SetFixedSize)
 
+    @Inputs.data
     @check_sql_input
     def set_data(self, data):
         """
@@ -97,54 +107,64 @@ class OWDistances(OWWidget):
 
     def commit(self):
         metric = METRICS[self.metric_idx]
-        self.send("Distances", self.compute_distances(metric, self.data))
+        self.Outputs.distances.send(self.compute_distances(metric, self.data))
 
     def compute_distances(self, metric, data):
+        def checks(metric, data):
+            if data is None:
+                return
+
+            if issparse(data.X) and not metric.supports_sparse:
+                self.Error.dense_metric_sparse_data()
+                return
+
+            if not any(a.is_continuous for a in data.domain.attributes):
+                self.Error.no_continuous_features()
+                return
+
+            needs_preprocessing = False
+            if any(a.is_discrete for a in self.data.domain.attributes):
+                self.Warning.ignoring_discrete()
+                needs_preprocessing = True
+
+            if not issparse(data.X) and bn.anynan(data.X):
+                self.Warning.imputing_data()
+                needs_preprocessing = True
+
+            if needs_preprocessing:
+                # removes discrete features and imputes data
+                data = distance._preprocess(data)
+
+            if not data.X.size:
+                self.Error.empty_data()
+                return
+
+            return data
+
         self.clear_messages()
 
+        data = checks(metric, data)
         if data is None:
             return
 
-        if issparse(data.X) and not metric.supports_sparse:
-            self.Error.dense_metric_sparse_data()
-            return
-
-        if not any(a.is_continuous for a in data.domain.attributes):
-            self.Error.no_continuous_features()
-            return
-
-        needs_preprocessing = False
-        if any(a.is_discrete for a in self.data.domain.attributes):
-            self.Warning.ignoring_discrete()
-            needs_preprocessing = True
-
-        if not issparse(data.X) and bn.anynan(data.X):
-            self.Warning.imputing_data()
-            needs_preprocessing = True
-
-        if needs_preprocessing:
-            # removes discrete features and imputes data
-            data = distance._preprocess(data)
-
-        if not data.X.size:
-            self.Error.empty_data()
-            return
-
-        if isinstance(metric, distance.MahalanobisDistance):
-            n, m = data.X.shape
-            if self.axis == 1:
-                n, m = m, n
-
         if isinstance(metric, distance.MahalanobisDistance):
             # Mahalanobis distance has to be trained before it can be used
             # to compute distances
             try:
-                metric.fit(data, axis=1 - self.axis)
+                metric = distance.MahalanobisDistance(data, axis=1 - self.axis)
             except (ValueError, MemoryError) as e:
                 self.Error.mahalanobis_error(e)
                 return
 
-        return metric(data, data, 1 - self.axis, impute=True)
+        try:
+            met = metric(data, data, 1 - self.axis, impute=True)
+        except ValueError as e:
+            self.Error.distances_value_error(e)
+            return
+        except MemoryError:
+            self.Error.distances_memory_error()
+            return
+        return met
 
     def _invalidate(self):
         self._checksparse()
