@@ -1,5 +1,4 @@
 import sys
-import warnings
 
 from xml.sax.saxutils import escape
 from itertools import chain
@@ -10,11 +9,13 @@ import numpy
 import scipy.spatial.distance
 
 from AnyQt.QtWidgets import (
-    QFormLayout, QHBoxLayout, QGroupBox, QToolButton, QActionGroup, QAction, QApplication,
-    QGraphicsLineItem
+    QFormLayout, QHBoxLayout, QGroupBox, QToolButton, QActionGroup, QAction,
+    QApplication, QGraphicsLineItem
 )
-from AnyQt.QtGui import QColor, QPen, QBrush, QPainter, QKeySequence, QCursor, QIcon
-from AnyQt.QtCore import Qt, QEvent
+from AnyQt.QtGui import (
+    QColor, QPen, QBrush, QPainter, QKeySequence, QCursor, QIcon
+)
+from AnyQt.QtCore import Qt, QTimer
 
 import pyqtgraph as pg
 import pyqtgraph.graphicsItems.ScatterPlotItem
@@ -29,16 +30,16 @@ from Orange.widgets import widget, gui, settings
 from Orange.widgets.utils import colorpalette, itemmodels
 from Orange.widgets.utils.sql import check_sql_input
 from Orange.canvas import report
-from Orange.widgets.widget import Msg, OWWidget
+from Orange.widgets.widget import Msg, OWWidget, Input, Output
 from Orange.widgets.utils.annotated_data import (create_annotated_table,
                                                  ANNOTATED_DATA_SIGNAL_NAME)
 
 
-def stress(X, D):
-    assert X.shape[0] == D.shape[0] == D.shape[1]
+def stress(X, distD):
+    assert X.shape[0] == distD.shape[0] == distD.shape[1]
     D1_c = scipy.spatial.distance.pdist(X, metric="euclidean")
     D1 = scipy.spatial.distance.squareform(D1_c, checks=False)
-    delta = D1 - D
+    delta = D1 - distD
     delta_sq = numpy.square(delta, out=delta)
     return delta_sq.sum(axis=0) / 2
 
@@ -52,9 +53,6 @@ def make_pen(color, width=1.5, style=Qt.SolidLine, cosmetic=False):
 class ScatterPlotItem(pg.ScatterPlotItem):
     Symbols = pyqtgraph.graphicsItems.ScatterPlotItem.Symbols
 
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-
     def paint(self, painter, option, widget=None):
         if self.opts["pxMode"]:
             painter.setRenderHint(QPainter.SmoothPixmapTransform, True)
@@ -70,12 +68,15 @@ class OWMDS(OWWidget):
     description = "Two-dimensional data projection by multidimensional " \
                   "scaling constructed from a distance matrix."
     icon = "icons/MDS.svg"
-    inputs = [("Data", Orange.data.Table, "set_data", widget.Default),
-              ("Distances", Orange.misc.DistMatrix, "set_disimilarity"),
-              ("Data Subset", Orange.data.Table, "set_subset_data")]
 
-    outputs = [("Selected Data", Orange.data.Table, widget.Default),
-               (ANNOTATED_DATA_SIGNAL_NAME, Orange.data.Table)]
+    class Inputs:
+        data = Input("Data", Orange.data.Table, default=True)
+        distances = Input("Distances", Orange.misc.DistMatrix)
+        data_subset = Input("Data Subset", Orange.data.Table)
+
+    class Outputs:
+        selected_data = Output("Selected Data", Orange.data.Table, default=True)
+        annotated_data = Output(ANNOTATED_DATA_SIGNAL_NAME, Orange.data.Table)
 
     #: Initialization type
     PCA, Random = 0, 1
@@ -112,6 +113,8 @@ class OWMDS(OWWidget):
     output_embedding_role = settings.Setting(2)
     autocommit = settings.Setting(True)
 
+    selection_indices = settings.Setting(None, schema_only=True)
+
     color_value = settings.ContextSetting("")
     shape_value = settings.ContextSetting("")
     size_value = settings.ContextSetting("")
@@ -133,6 +136,8 @@ class OWMDS(OWWidget):
         no_attributes = Msg("Data has no attributes")
         mismatching_dimensions = \
             Msg("Data and distances dimensions do not match.")
+        out_of_memory = Msg("Out of memory")
+        optimization_error = Msg("Error during optimization\n{}")
 
     def __init__(self):
         super().__init__()
@@ -163,6 +168,9 @@ class OWMDS(OWWidget):
         self._effective_matrix = None
 
         self.__update_loop = None
+        # timer for scheduling updates
+        self.__timer = QTimer(self, singleShot=True, interval=0)
+        self.__timer.timeout.connect(self.__next_step)
         self.__state = OWMDS.Waiting
         self.__in_next_step = False
         self.__draw_similar_pairs = False
@@ -352,6 +360,7 @@ class OWMDS(OWWidget):
 
         self._initialize()
 
+    @Inputs.data
     @check_sql_input
     def set_data(self, data):
         """Set the input data set.
@@ -377,6 +386,7 @@ class OWMDS(OWWidget):
             self._invalidated = True
         self._selection_mask = None
 
+    @Inputs.distances
     def set_disimilarity(self, matrix):
         """Set the dissimilarity (distance) matrix.
 
@@ -399,6 +409,7 @@ class OWMDS(OWWidget):
         self._invalidated = True
         self._selection_mask = None
 
+    @Inputs.data_subset
     def set_subset_data(self, subset_data):
         """Set a subset of `data` input to highlight in the plot.
 
@@ -444,15 +455,15 @@ class OWMDS(OWWidget):
             self._legend_item = None
 
     def update_controls(self):
-        if self.data is None and getattr(self.matrix, 'axis', 1) == 0:
-            # Column-wise distances
-            attr = "Attribute names"
+        if self.data is None:
+            axis = getattr(self.matrix, 'axis', 1)
+            attr = ["Column labels", "Row labels"][axis]
             self.labelvar_model[:] = ["No labels", attr]
-            self.shapevar_model[:] = ["Same shape", attr]
-            self.colorvar_model[:] = ["Same solor", attr]
+            self.shapevar_model[:] = ["Same shape"]
+            self.colorvar_model[:] = ["Same color"]
 
-            self.color_value = attr
-            self.shape_value = attr
+            self.color_value = "Same color"
+            self.shape_value = "Same shape"
         else:
             domain = self.data.domain
             all_vars = list(filter_visible(domain.variables + domain.metas))
@@ -514,6 +525,7 @@ class OWMDS(OWWidget):
 
         self.update_controls()
         self.openContext(self.data)
+        self.select_indices(self.selection_indices)
 
     def _toggle_run(self):
         if self.__state == OWMDS.Running:
@@ -590,8 +602,8 @@ class OWMDS(OWWidget):
         MDS points, `stress` is the current stress and `progress` a float
         ratio (0 <= progress <= 1)
 
-        If an existing update loop is already in palace it is interrupted
-        (closed).
+        If an existing update coroutine loop is already in palace it is
+        interrupted (i.e. closed).
 
         .. note::
             The `loop` must not explicitly yield control flow to the event
@@ -606,23 +618,30 @@ class OWMDS(OWWidget):
         self.__update_loop = loop
 
         if loop is not None:
+            self.setBlocking(True)
             self.progressBarInit(processEvents=None)
             self.setStatusMessage("Running")
             self.runbutton.setText("Stop")
             self.__state = OWMDS.Running
-            QApplication.postEvent(self, QEvent(QEvent.User))
+            self.__timer.start()
         else:
+            self.setBlocking(False)
             self.setStatusMessage("")
             self.runbutton.setText("Start")
             self.__state = OWMDS.Finished
+            self.__timer.stop()
 
     def __next_step(self):
         if self.__update_loop is None:
             return
 
+        assert not self.__in_next_step
+        self.__in_next_step = True
+
         loop = self.__update_loop
+        self.Error.out_of_memory.clear()
         try:
-            embedding, stress, progress = next(self.__update_loop)
+            embedding, _, progress = next(self.__update_loop)
             assert self.__update_loop is loop
         except StopIteration:
             self.__set_update_loop(None)
@@ -630,31 +649,23 @@ class OWMDS(OWWidget):
             self.__draw_similar_pairs = True
             self._update_plot()
             self.plot.autoRange(padding=0.1, items=[self._scatter_item])
+        except MemoryError:
+            self.Error.out_of_memory()
+            self.__set_update_loop(None)
+            self.__draw_similar_pairs = True
+        except Exception as exc:
+            self.Error.optimization_error(str(exc))
+            self.__set_update_loop(None)
+            self.__draw_similar_pairs = True
         else:
             self.progressBarSet(100.0 * progress, processEvents=None)
             self.embedding = embedding
             self._update_plot()
             self.plot.autoRange(padding=0.1, items=[self._scatter_item])
             # schedule next update
-            QApplication.postEvent(
-                self, QEvent(QEvent.User), Qt.LowEventPriority)
+            self.__timer.start()
 
-    def customEvent(self, event):
-        if event.type() == QEvent.User and self.__update_loop is not None:
-            if not self.__in_next_step:
-                self.__in_next_step = True
-                try:
-                    self.__next_step()
-                finally:
-                    self.__in_next_step = False
-            else:
-                warnings.warn(
-                    "Re-entry in update loop detected. "
-                    "A rogue `proccessEvents` is on the loose.",
-                    RuntimeWarning)
-                # re-schedule the update iteration.
-                QApplication.postEvent(self, QEvent(QEvent.User))
-        return super().customEvent(event)
+        self.__in_next_step = False
 
     def __invalidate_embedding(self):
         # reset/invalidate the MDS embedding, to the default initialization
@@ -738,16 +749,20 @@ class OWMDS(OWWidget):
     def _setup_plot(self):
         have_data = self.data is not None
         have_matrix_transposed = self.matrix is not None and not self.matrix.axis
-        plotstyle = mdsplotutils.plotstyle
+        plotstyle = Mdsplotutils.plotstyle
 
         size = self._effective_matrix.shape[0]
 
-        def column(data, variable):
+        def column(data, variable, dtype=None):
             a, _ = data.get_column_view(variable)
+            if dtype is not None:
+                a = a.astype(dtype)
             return a.ravel()
 
         def attributes(matrix):
-            return matrix.row_items.domain.attributes
+            if matrix.row_items and matrix.row_items.domain:
+                return matrix.row_items.domain.attributes
+            return [x + 1 for x in range(len(matrix))]
 
         def scale(a):
             dmin, dmax = numpy.nanmin(a), numpy.nanmax(a)
@@ -771,7 +786,7 @@ class OWMDS(OWWidget):
             if self._selection_mask is not None:
                 pointflags = numpy.where(
                     self._selection_mask,
-                    mdsplotutils.Selected, mdsplotutils.NoFlags)
+                    Mdsplotutils.Selected, Mdsplotutils.NoFlags)
             else:
                 pointflags = None
 
@@ -786,27 +801,15 @@ class OWMDS(OWWidget):
                 else:
                     palette = None
 
-                color_data = mdsplotutils.color_data(
+                color_data = Mdsplotutils.color_data(
                     self.data, color_var, plotstyle=plotstyle)
                 color_data = numpy.hstack(
                     (color_data,
                      numpy.full((len(color_data), 1), self.symbol_opacity,
                                 dtype=float))
                 )
-                pen_data = mdsplotutils.pen_data(color_data * 0.8, pointflags)
-                brush_data = mdsplotutils.brush_data(color_data)
-            elif have_matrix_transposed and \
-                    self.colorvar_model[color_index] == 'Attribute names':
-                attr = attributes(self.matrix)
-                palette = colorpalette.ColorPaletteGenerator(len(attr))
-                color_data = [palette.getRGB(i) for i in range(len(attr))]
-                color_data = numpy.hstack((
-                    color_data,
-                    numpy.full((len(color_data), 1), self.symbol_opacity,
-                               dtype=float))
-                )
-                pen_data = mdsplotutils.pen_data(color_data * 0.8, pointflags)
-                brush_data = mdsplotutils.brush_data(color_data)
+                pen_data = Mdsplotutils.pen_data(color_data * 0.8, pointflags)
+                brush_data = Mdsplotutils.brush_data(color_data)
             else:
                 pen_data = make_pen(QColor(Qt.darkGray), cosmetic=True)
                 if self._selection_mask is not None:
@@ -814,7 +817,7 @@ class OWMDS(OWWidget):
                         [pen_data, plotstyle.selected_pen])
                     pen_data = pen_data[self._selection_mask.astype(int)]
                 else:
-                    pen_data = numpy.full(self._effective_matrix.dim, pen_data,
+                    pen_data = numpy.full(self._effective_matrix.shape[0], pen_data,
                                           dtype=object)
                 brush_data = numpy.full(
                     size, pg.mkColor((192, 192, 192, self.symbol_opacity)),
@@ -835,17 +838,10 @@ class OWMDS(OWWidget):
                 symbols = numpy.array(list(Symbols.keys()))
 
                 shape_var = self.shapevar_model[shape_index]
-                data = column(self.data, shape_var).astype(numpy.float)
+                data = column(self.data, shape_var, dtype=numpy.float)
                 data = data % (len(Symbols) - 1)
                 data[numpy.isnan(data)] = len(Symbols) - 1
                 shape_data = symbols[data.astype(int)]
-            elif have_matrix_transposed and \
-                    self.shapevar_model[shape_index] == 'Attribute names':
-                Symbols = ScatterPlotItem.Symbols
-                symbols = numpy.array(list(Symbols.keys()))
-                attr = [i % (len(Symbols) - 1)
-                        for i, _ in enumerate(attributes(self.matrix))]
-                shape_data = symbols[attr]
             else:
                 shape_data = "o"
             self._shape_data = shape_data
@@ -861,9 +857,10 @@ class OWMDS(OWWidget):
                 size_data = MinPointSize + size_data * point_size
             elif have_data and size_index > 0:
                 size_var = self.sizevar_model[size_index]
-                size_data = column(self.data, size_var)
+                size_data = column(self.data, size_var, dtype=float)
                 size_data = scale(size_data)
                 size_data = MinPointSize + size_data * point_size
+                size_data[numpy.isnan(size_data)] = 1  # maybe 0?
             else:
                 size_data = point_size
             self._size_data = size_data
@@ -876,10 +873,10 @@ class OWMDS(OWWidget):
                 label_data = [label_var.str_val(val) for val in label_data]
                 label_items = [pg.TextItem(text, anchor=(0.5, 0), color=0.0)
                                for text in label_data]
-            elif have_matrix_transposed and \
-                    self.labelvar_model[label_index] == 'Attribute names':
+            elif self.matrix is not None and label_index:
                 attr = attributes(self.matrix)
-                label_items = [pg.TextItem(str(text), anchor=(0.5, 0))
+                label_items = [pg.TextItem(str(text), anchor=(0.5, 0),
+                                           color=0.0)
                                for text in attr]
             else:
                 label_items = None
@@ -971,7 +968,7 @@ class OWMDS(OWWidget):
         if shape_var is not None or \
                 (color_var is not None and color_var.is_discrete):
 
-            legend_data = mdsplotutils.legend_data(
+            legend_data = Mdsplotutils.legend_data(
                 color_var, shape_var, plotstyle=plotstyle)
 
             for color, symbol, text in legend_data:
@@ -1007,7 +1004,7 @@ class OWMDS(OWWidget):
                 metas += embedding.domain.attributes
 
             domain = Orange.data.Domain(attrs, class_vars, metas)
-            output = Orange.data.Table.from_table(domain, self.data)
+            output = self.data.transform(domain)
 
             if self.output_embedding_role == OWMDS.AttrRole:
                 output.X[:] = embedding.X
@@ -1019,11 +1016,11 @@ class OWMDS(OWWidget):
         if output is not None and self._selection_mask is not None and \
                 numpy.any(self._selection_mask):
             subset = output[self._selection_mask]
+            self.selection_indices = numpy.flatnonzero(self._selection_mask)
         else:
-            subset = None
-        self.send("Selected Data", subset)
-        self.send(ANNOTATED_DATA_SIGNAL_NAME,
-                  create_annotated_table(output, self._selection_mask))
+            self.selection_indices = subset = None
+        self.Outputs.selected_data.send(subset)
+        self.Outputs.annotated_data.send(create_annotated_table(output, self._selection_mask))
 
     def onDeleteWidget(self):
         super().onDeleteWidget()
@@ -1052,7 +1049,7 @@ class OWMDS(OWWidget):
         self.select_indices(indices, QApplication.keyboardModifiers())
 
     def select_indices(self, indices, modifiers=Qt.NoModifier):
-        if self.data is None:
+        if self.data is None or indices is None:
             return
 
         if self._selection_mask is None or \
@@ -1134,18 +1131,18 @@ from Orange.widgets.visualize.owlinearprojection import \
 from Orange.widgets.visualize.owlinearprojection import plotutils
 
 
-class namespace(namespace):
+class Namespace(namespace):
     def updated(self, **kwargs):
         ns = self.__dict__.copy()
         ns.update(**kwargs)
-        return namespace(**ns)
+        return Namespace(**ns)
 
 
-class mdsplotutils(plotutils):
+class Mdsplotutils(plotutils):
     NoFlags, Selected, Highlight = 0, 1, 2
     NoFill, Filled = 0, 1
 
-    plotstyle = namespace(
+    plotstyle = Namespace(
         selected_pen=make_pen(Qt.yellow, width=3, cosmetic=True),
         highligh_pen=QPen(Qt.blue, 1),
         selected_brush=None,
@@ -1180,13 +1177,13 @@ class mdsplotutils(plotutils):
             N = numpy.count_nonzero(mask)
 
         if plotstyle is None:
-            plotstyle = mdsplotutils.plotstyle
+            plotstyle = Mdsplotutils.plotstyle
 
         if var is None:
             col = numpy.zeros(N, dtype=float)
             color_data = numpy.full(N, plotstyle.default_color, dtype=object)
         elif var.is_primitive():
-            col = mdsplotutils.column_data(table, var, mask)
+            col = Mdsplotutils.column_data(table, var, mask)
             if var.is_discrete:
                 palette = plotstyle.discrete_palette
                 if len(var.values) >= palette.number_of_colors:
@@ -1198,28 +1195,28 @@ class mdsplotutils(plotutils):
                 color_data = plotutils.continuous_colors(
                     col, palette=plotstyle.continuous_palette)
         else:
-            raise TypeError("Discrete/Continuous variable or None expected.")
+            raise TypeError("Categorical/Numeric variable or None expected.")
 
         return color_data
 
     @staticmethod
     def pen_data(basecolors, flags=None, plotstyle=None):
         if plotstyle is None:
-            plotstyle = mdsplotutils.plotstyle
+            plotstyle = Mdsplotutils.plotstyle
 
         pens = numpy.array(
-            [mdsplotutils.make_pen(QColor(*rgba), width=1)
+            [Mdsplotutils.make_pen(QColor(*rgba), width=1)
              for rgba in basecolors],
             dtype=object)
 
         if flags is None:
             return pens
 
-        selected_mask = flags & mdsplotutils.Selected
+        selected_mask = flags & Mdsplotutils.Selected
         if numpy.any(selected_mask):
             pens[selected_mask.astype(bool)] = plotstyle.selected_pen
 
-        highlight_mask = flags & mdsplotutils.Highlight
+        highlight_mask = flags & Mdsplotutils.Highlight
         if numpy.any(highlight_mask):
             pens[highlight_mask.astype(bool)] = plotstyle.hightlight_pen
 
@@ -1228,17 +1225,17 @@ class mdsplotutils(plotutils):
     @staticmethod
     def brush_data(basecolors, flags=None, plotstyle=None):
         if plotstyle is None:
-            plotstyle = mdsplotutils.plotstyle
+            plotstyle = Mdsplotutils.plotstyle
 
         brush = numpy.array(
-            [mdsplotutils.make_brush(QColor(*c))
+            [Mdsplotutils.make_brush(QColor(*c))
              for c in basecolors],
             dtype=object)
 
         if flags is None:
             return brush
 
-        fill_mask = flags & mdsplotutils.Filled
+        fill_mask = flags & Mdsplotutils.Filled
 
         if not numpy.all(fill_mask):
             brush[~fill_mask] = QBrush(Qt.NoBrush)
@@ -1247,7 +1244,7 @@ class mdsplotutils(plotutils):
     @staticmethod
     def shape_data(table, var, mask=None, plotstyle=None):
         if plotstyle is None:
-            plotstyle = mdsplotutils.plotstyle
+            plotstyle = Mdsplotutils.plotstyle
 
         N = len(table)
         if mask is not None:
@@ -1257,7 +1254,7 @@ class mdsplotutils(plotutils):
         if var is None:
             return numpy.full(N, "o", dtype=object)
         elif var.is_discrete:
-            shape_data = mdsplotutils.column_data(table, var, mask)
+            shape_data = Mdsplotutils.column_data(table, var, mask)
             maxsymbols = len(plotstyle.symbols) - 1
             validmask = numpy.isfinite(shape_data)
             shape = shape_data % (maxsymbols - 1)
@@ -1275,7 +1272,7 @@ class mdsplotutils(plotutils):
     @staticmethod
     def size_data(table, var, mask=None, plotstyle=None):
         if plotstyle is None:
-            plotstyle = mdsplotutils.plotstyle
+            plotstyle = Mdsplotutils.plotstyle
 
         N = len(table)
         if mask is not None:
@@ -1285,8 +1282,8 @@ class mdsplotutils(plotutils):
         if var is None:
             return numpy.full(N, plotstyle.point_size, dtype=float)
         else:
-            size_data = mdsplotutils.column_data(table, var, mask)
-            size_data = mdsplotutils.normalized(size_data)
+            size_data = Mdsplotutils.column_data(table, var, mask)
+            size_data = Mdsplotutils.normalized(size_data)
             size_mask = numpy.isnan(size_data)
             size_data = size_data * plotstyle.point_size + \
                         plotstyle.min_point_size
@@ -1300,7 +1297,7 @@ class mdsplotutils(plotutils):
     @staticmethod
     def legend_data(color_var=None, shape_var=None, plotstyle=None):
         if plotstyle is None:
-            plotstyle = mdsplotutils.plotstyle
+            plotstyle = Mdsplotutils.plotstyle
 
         if color_var is not None and not color_var.is_discrete:
             color_var = None
@@ -1345,7 +1342,9 @@ class mdsplotutils(plotutils):
         return QBrush(color, )
 
 
-def main_test(argv=sys.argv):
+def main_test(argv=None):
+    if argv is None:
+        argv = sys.argv
     import gc
     app = QApplication(list(argv))
     argv = app.arguments()
