@@ -1,12 +1,36 @@
 import re
 import numpy as np
-from Orange.data import Table, Domain, DiscreteVariable
+from Orange.data import Domain, DiscreteVariable
 
 ANNOTATED_DATA_SIGNAL_NAME = "Data"
 ANNOTATED_DATA_FEATURE_NAME = "Selected"
 
 
-def _get_next_name(names, name):
+def add_columns(domain, attributes=(), class_vars=(), metas=()):
+    """Construct a new domain with new columns added to the specified place
+
+    Parameters
+    ----------
+    domain : Domain
+        source domain
+    attributes
+        list of variables to append to attributes from source domain
+    class_vars
+        list of variables to append to class_vars from source domain
+    metas
+        list of variables to append to metas from source domain
+
+    Returns
+    -------
+    Domain
+    """
+    attributes = domain.attributes + tuple(attributes)
+    class_vars = domain.class_vars + tuple(class_vars)
+    metas = domain.metas + tuple(metas)
+    return Domain(attributes, class_vars, metas)
+
+
+def get_next_name(names, name):
     """
     Returns next 'possible' attribute name. The name should not be duplicated
     and is generated using name parameter, appended by smallest possible index.
@@ -35,14 +59,11 @@ def create_annotated_table(data, selected_indices):
     if data is None:
         return None
     names = [var.name for var in data.domain.variables + data.domain.metas]
-    name = _get_next_name(names, ANNOTATED_DATA_FEATURE_NAME)
-    metas = data.domain.metas + (DiscreteVariable(name, ("No", "Yes")),)
-    domain = Domain(data.domain.attributes, data.domain.class_vars, metas)
+    name = get_next_name(names, ANNOTATED_DATA_FEATURE_NAME)
+    domain = add_columns(data.domain, metas=[DiscreteVariable(name, ("No", "Yes"))])
     annotated = np.zeros((len(data), 1))
     if selected_indices is not None:
         annotated[selected_indices] = 1
-    table = Table(domain, data.X, data.Y,
-                  metas=np.hstack((data.metas, annotated)))
-    table.attributes = data.attributes
-    table.ids = data.ids
+    table = data.transform(domain)
+    table[:, name] = annotated
     return table
